local folder, core = ...

_G["TR"] = core

local _ --so FindGloabls doesn't nag me.
local LibStub = LibStub
local Debug

local curseVersion = GetAddOnMetadata(folder, "X-Curse-Packaged-Version")
local curseRevision = GetAddOnMetadata(folder, "X-Curse-Packaged-Revision") -- 189
if curseVersion and curseRevision and curseVersion ~= "r"..curseRevision then
	curseVersion = curseVersion.." (r"..curseRevision..")" --alpha versions will show "r99", tagged versions will show "1.2.3beta (r99)"
end



core.title		= GetAddOnMetadata(folder, "Title")
core.version	= curseVersion or "[dev]"
core.titleFull	= core.title.." "..(curseVersion and "v"..tostring(core.version) or tostring(core.version))

core.addonDir = "Interface\\AddOns\\"..folder.."\\"

local timerFrame = CreateFrame("Frame")
LibStub("AceAddon-3.0"):NewAddon(core, folder, "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceComm-3.0", "AceSerializer-3.0", "AceTimer-3.0") 

core.L = LibStub("AceLocale-3.0"):GetLocale(folder, true)
local L = core.L

local commPrefixTotemLoc	= "TR_L"--New totem info
local commPrefixRemoveTotem	= "TR_K"--Remove totem.

core.userName = ""
local timers = {
	distTimer = {},
}

local db
local P --db.profile

local defaultSettings = {
	profile = {
		totemColour= {},
		totemOpts = {},
	},
}
local activeTotems = {}
local activeCasters = {} --shaman who've cast totems.
local totemInfo

core.defaultSettings = defaultSettings
core.activeTotems = activeTotems
core.activeCasters = activeCasters

local regEvents = {
	"PLAYER_TOTEM_UPDATE",
	"COMBAT_LOG_EVENT_UNFILTERED",
	"UNIT_HEALTH",
	"CVAR_UPDATE",
	"MINIMAP_UPDATE_ZOOM",
}

--Option windows
local coreOpts
do
	local config, dialog
	function core:OnInitialize()
		Debug = self.Debug
		totemInfo = self.totemInfo
	
		db = LibStub("AceDB-3.0"):New("TR_DB", core.defaultSettings, true)
		self.CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)
		db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
		db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
		db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
		db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
		self.db = db
		self:RegisterChatCommand("tr", "MySlashProcessorFunc")
		
		self:CreateTotemDistanceFrames()
--~ 		config = LibStub("AceConfig-3.0")
--~ 		dialog = LibStub("AceConfigDialog-3.0")
--~ 		config:RegisterOptionsTable(folder, self.CoreOptionsTable)
--~ 		coreOpts = dialog:AddToBlizOptions(folder, self.titleFull)
	
		
	end
end

do
	local InterfaceOptionsFrame_OpenToCategory = InterfaceOptionsFrame_OpenToCategory
	----------------------------------------------
	function core:MySlashProcessorFunc(input)	--
	-- /da function brings up the UI options.	--
	----------------------------------------------
--~ 		InterfaceOptionsFrame_OpenToCategory(coreOpts)
		self:OpenOptionsFrame()
	end
end

local timerUpdate
do
	local GetPlayerMapPosition = GetPlayerMapPosition
	local GetPlayerFacing = GetPlayerFacing
	local MiniMapCompassRing = MiniMapCompassRing
	
	local pairs = pairs
	local sX, sY
	local sX, sY
	local lastFacing, facing = 0
	--------------------------------------------------------------------------
	core.updateThrottle = 0.05  --Update minimap location 20x a second.		--
	local lastX, lastY = 0,0												--
	function timerUpdate(self, elapsed)								--
	-- Update our minimap rings. 											--
	-- I'm using a OnUpdate timer instead of AceTimer because AceTimer 		--
	-- doesn't fire fast enough to make the minimap icons move smoothly.	--
	--------------------------------------------------------------------------
		self.lastUpdate = (self.lastUpdate or 0) + elapsed
		if self.lastUpdate > core.updateThrottle then
			self.lastUpdate = 0
	
			--Make sure we've moved, else we waste CPU cycles not changing anything.
			sX, sY = GetPlayerMapPosition("player")
			if lastX == sX and lastY == sY then
			
				if GetPlayerFacing then
					facing = GetPlayerFacing()
				else
					facing = -MiniMapCompassRing:GetFacing()
				end
				if facing == lastFacing then
					return
				end
				lastFacing = facing
			end
			lastX = sX
			lastY = sY 
	
			sX, sY = GetPlayerMapPosition("player")
			for totemGUID in pairs(activeTotems) do 
				core:TotemUpdate(totemGUID, sX, sY)
			end
			
		end
	end
end

do
	local pairs = pairs
	local GetUnitName = GetUnitName
	local _G = _G
	
	function core:OnEnable()
		P = db.profile
	
		for i, event in pairs(regEvents) do 
			self:RegisterEvent(event)
		end
		
		self.userName = GetUnitName("player")
	
		timerFrame:SetScript("OnUpdate", timerUpdate)
	
		--[[	
		if P.showOnMinimap == false then
			self.updateThrottle = 0.25 --User's probably running AVR. We just do inside/outside check. This doesn't need to be done rapidly.
		else
			self.updateThrottle = 0.05
		end
		]]
		
		self:RegisterComm(commPrefixTotemLoc, "IncComm")
		self:RegisterComm(commPrefixRemoveTotem, "IncComm")
		
		for i=1, 4 do 
			if _G["TotemFrameTotem"..i.."Distance"] then
				if self.db.profile.showTotemDistance == true then
					_G["TotemFrameTotem"..i.."Distance"]:Show()
				end
			end
		end
		
	--~ 	Debug("OnEnable","core.lua")
	end

	function core:OnDisable()
	--~ 	HideAllTotems()
	--~ 	Debug("OnDisable","core")
	
		--Kill our OnUpdate timer.
		timerFrame:SetScript("OnUpdate", nil)
		
		for i=1, 4 do 
			if _G["TotemFrameTotem"..i.."Distance"] then
				_G["TotemFrameTotem"..i.."Distance"]:Hide()
			end
		end
	end
end


----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end

local function RemoveTotem(totemGUID)
	if timers[totemGUID] then
		core:CancelTimer(timers[totemGUID], true);
	end

	if activeTotems[totemGUID] then
--~ 		core:RemoveTotemFromAVR(totemGUID)
		core:RemoveTotemFromMinimap(totemGUID)
		
		activeTotems[totemGUID] = nil
	end
	
end

local BroadcastRemoveTotem
do
	local GetNumPartyMembers = GetNumPartyMembers
	local IsInInstance = IsInInstance
	local GetNumRaidMembers = GetNumRaidMembers
	local str
	local iActive, iType
	----------------------------------------------------------------------------------
	function BroadcastRemoveTotem(totemGUID)									--
	-- Sends the totemGUID to group for others to remove if from their minimap.		--
	-- This is needed cause UNIT_DEAD doesn't always fire if we're out of range.	--
	----------------------------------------------------------------------------------
		if GetNumPartyMembers() > 0 then
			str = core:Serialize(totemGUID)
	--~ 		core:SendCommMessage(commPrefixRemoveTotem, str, "GUILD", nil, "ALERT") --Debugging
	
			iActive, iType = IsInInstance()
			if iActive and iType == "pvp" then
				--We're in a battlefield. 
				core:SendCommMessage(commPrefixRemoveTotem, str, "BATTLEGROUND", nil, "BULK")
			else
				core:SendCommMessage(commPrefixRemoveTotem, str, GetNumRaidMembers() > 1 and "raid" or "party", nil, "BULK")
			end
		end
	end
end

do
	local GetTotemInfo = GetTotemInfo
	local pairs = pairs
	local haveTotem, totemName, startTime, duration
	------------------------------------------------------
	function core:PLAYER_TOTEM_UPDATE(event, totemSlot)	--
	-- We only track when our totems disapear here.		-- 
	-- Plus we broadcast that our totem is gone.		--
	-- New totems are done in the combatlog.			--
	------------------------------------------------------
		haveTotem, totemName, startTime, duration = GetTotemInfo(totemSlot)
	
		if startTime == 0 then
			for GUID, data in pairs(activeTotems) do 
				if data.caster == self.userName and data.totemSlot == totemSlot then
					RemoveTotem(GUID)
					if P.broadcastTotemLocation == true then
						BroadcastRemoveTotem(GUID)
					end
					break
				end
			end
		end
	end
end

local RemoveCastersTotems
do
	local pairs = pairs
	------------------------------------------
	function RemoveCastersTotems(caster)	--
	-- Remove all totems from a shaman.		--
	------------------------------------------
		for totemGUID, data in pairs(activeTotems) do 
			if data.caster == caster then
				RemoveTotem(totemGUID)
			end
		end
		activeCasters[caster] = nil
	end
end

local BroadcastTotemLoc
do
	local GetNumPartyMembers = GetNumPartyMembers
	local IsInInstance = IsInInstance
	local GetNumRaidMembers = GetNumRaidMembers
	
	local str
	local iActive, iType 
	----------------------------------------------------------------------------------------------------------
	function BroadcastTotemLoc(totemGUID, spellID, tC, tZ, tX, tY, zoneMap)							--
	-- Send the precise coords of our totems so others can update the locs of our totems on their minimap.	--
	----------------------------------------------------------------------------------------------------------
		if GetNumPartyMembers() > 0 then
			tX = core:Round(tX,8) --shrink the outgoing message by 15 characters.
			tY = core:Round(tY,8)
			str = core:Serialize(totemGUID, spellID, tC, tZ, tX, tY, zoneMap)
	--~ 		Debug("BroadcastTotemLoc 2", str)
	
			iActive, iType = IsInInstance()
			if iActive and iType == "pvp" then
				core:SendCommMessage(commPrefixTotemLoc, str, "BATTLEGROUND", nil, "ALERT")
			else
				core:SendCommMessage(commPrefixTotemLoc, str, GetNumRaidMembers() > 1 and "raid" or "party", nil, "ALERT")
			end
		end
		
	--~ 	tX = core:Round(tX,8) --shrink the outgoing message by 15 characters.
	--~ 	tY = core:Round(tY,8)
	--~ 	local str = core:Serialize(totemGUID, spellID, tC, tZ, tX, tY, zoneMap)
	--~ 	Debug("BroadcastTotemLoc A",totemGUID, spellID, tC, tZ, tX, tY, zoneMap)
	--~ 	Debug("BroadcastTotemLoc B",str)
	end
end

local RemoveCastersOldTotems
do
	local pairs = pairs
	----------------------------------------------------------
	function RemoveCastersOldTotems(caster, totemSlot)--
	-- Remove last known totem for a caster's totem slot.	--
	----------------------------------------------------------
		for totemGUID, data in pairs(activeTotems) do 
			if data.caster == caster and data.totemSlot == totemSlot then
	--~ 			Debug("RemoveCastersOldTotems", caster, totemSlot, totemGUID)
				RemoveTotem(totemGUID)
				return
			end
		end
	end
end

do
	local GetSpellInfo = GetSpellInfo
	local UnitInParty = UnitInParty 
	local UnitInRaid = UnitInRaid
	local GetCurrentMapContinent = GetCurrentMapContinent
	local GetCurrentMapZone = GetCurrentMapZone
	local GetPlayerMapPosition = GetPlayerMapPosition
	
	local totemName
	local zoneMap 
	local range
	local duration
	local zW, zH
	local myMap
	local sX, sY 
	
	local LibTalentQuery = LibStub("LibTalentQuery-1.0") 
	local LibGroupTalents = LibStub("LibGroupTalents-1.0") 
	local reachTalentName = GetSpellInfo(86935) -- Totemic Reach 
	local elementalTalentName = GetSpellInfo(28999) -- Elemental Reach 
	local focusTalentName = GetSpellInfo(16173) -- Totemic Focus 
	local reach, elementalReach, focus
	
	function core:AddTotem(totemGUID, spellID, caster, tC, tZ, tX, tY, precise, totemSlot, zoneMap)	--
		if not totemInfo[spellID] then
			return
		end
		local totemSlot = totemSlot or totemInfo[spellID].slot or 1
		RemoveCastersOldTotems(caster, totemSlot)
		
		zoneMap = zoneMap or self:ZoneIDToMap(tC, tZ)
		
		totemName = totemInfo[spellID].name or GetSpellInfo(spellID)
	
	--~ 	Debug("AddTotem", 1, totemName)
		if not P.totemOpts[totemName] then
			return
		end
	--~ 	Debug("AddTotem", 2, totemName)
	
		if P.totemOpts[totemName].shown == 4 then --never show totem
			return
		end
		if (caster == self.userName) or 
		(P.totemOpts[totemName].shown == 2 and UnitInParty(caster)) or 
		(P.totemOpts[totemName].shown == 1 and (UnitInRaid(caster) or UnitInParty(caster))) then
			activeCasters[caster] = true
	--~ 		Debug("AddTotem", 3, totemName)
				
			range = totemInfo[spellID].range or 0
			
			if LibGroupTalents then
				--belechannas's fix to include Totemic Reach and Elemental Reach's range increase.
				reach = LibGroupTalents:UnitHasTalent(caster, reachTalentName)
				range = range * (1 + (reach or 0) * 0.15)
				-- Searing Totem + Elemental Mastery
				if ( spellID == 3599 ) then
					elementalReach = LibGroupTalents:UnitHasTalent(caster, elementalTalentName)
					if (elementalReach == 1) then
						range = range + 7
					else
						if (elementalReach == 2) then
							range = range + 15
						end
					end
				end 
			end
			
			
			
			if precise == false and range > self.spawnDistance then
				range = range - self.spawnDistance --We don't know the precise coords of the totem. So we remove 3 yards from the totem's range to make sure we'll be inside it. 
			end
			
			if not activeTotems[totemGUID] then
	--~ 			Debug("AddTotem", 4, totemName)
				duration = totemInfo[spellID].duration or 5
				
				if LibGroupTalents then
					--belechannas's fix to include Totemic Focus's duration increase.
					focus = LibGroupTalents:UnitHasTalent(caster, focusTalentName) 
					duration = duration * (1 + (focus or 0) * 0.20) 
				end
				
				zW, zH = self:GetZoneSize()

				activeTotems[totemGUID] = {
					caster	= caster,
		
					--coords
					tC = tC,
					tZ = tZ,
					tX = tX,
					tY = tY,
		
					zW = zW, --Zone width
					zH = zH, --Zone height
					
					range	= range,
					precise	= precise,
					totemSlot = totemSlot,
	--~ 				death = GetTime() + duration,--Remove the icon at this time, just in case UNIT_DEAD doesn't see it.
					totemName = totemName, 
					zoneMap = zoneMap,
					
				}
	
				--Create a minimap icon.
				activeTotems[totemGUID].minimapIcon = self:AddTotemToMinimap(totemGUID, spellID, totemSlot, range, precise)
	
				--Start a timer with the totem's duration. At that time remove the totem.
				self:ScheduleTimer("TotemAgeDeath", duration, totemGUID)
			end
	
			myMap = self:GetCurrentMap()
			
			--I should have just used zoneMap in the orignal version. Continent and Zone is here for backwords compatibility.
			if zoneMap == myMap or tC == GetCurrentMapContinent() and tZ == GetCurrentMapZone() then
	--~ 			Debug("AddTotem", 5, totemName)
	--~ 			if P.AVRdisplay == true then
	--~ 				self:AddTotemToAVR(totemGUID, tX, tY, totemSlot, spellID, range, precise, caster, totemName) --
	--~ 			end
					
					
				if activeTotems[totemGUID].minimapIcon then
					activeTotems[totemGUID].minimapIcon:Show()
				end
				sX, sY = GetPlayerMapPosition("player")
				self:TotemUpdate(totemGUID, sX, sY)
			end
		
		end
		
	--~ 	Debug("AddTotem", 5, totemName)
		
		if caster ~= self.userName and not timers.distTimer[totemGUID] then
			--Every ten seconds check the distance of the caster to their totems. If they're 100 yards away, remove the totem.
			timers.distTimer[totemGUID] = self:ScheduleRepeatingTimer("CheckCasterDistance", 10, totemGUID)
		end
	end
end

do
	local GetPlayerMapPosition = GetPlayerMapPosition
	local pairs = pairs
	local sX, sY 
	--Used in minimap.lua MINIMAP_UPDATE_ZOOM
	function core:ForceUpdate()
		sX, sY = GetPlayerMapPosition("player")
		for totemGUID in pairs(activeTotems) do 
			core:TotemUpdate(totemGUID, sX, sY)
		end
	end
end

--~ function core:AddDistToFrame(slot, 

do
	local pairs = pairs
	local totemData
	local range
	local zW, zH
	local tX, tY
	local dist, xDist, yDist
	local insideRing
	local _G = _G
	
	local tFrame, fName
	--------------------------------------------------
	function core:TotemUpdate(totemGUID, sX, sY)	--
	-- Main update totem function. 					--
	--------------------------------------------------
		totemData = activeTotems[totemGUID]
		range = totemData.range
		
		--Zone width and height
		zW, zH = totemData.zW, totemData.zH
		
		--Totem's map coords.
		tX, tY = totemData.tX, totemData.tY
		
		--Convert totem to yard coords.
		tX = zW * tX
		tY = zH * tY
		
		--Our coords in yard coords.
		sX = zW * sX
		sY = zH * sY
	
		dist, xDist, yDist = self:ComputeDistance(sX, sY, tX, tY)
		
		totemData.dist = dist
		
		insideRing = true
		

		
		--Check if we're inside or outside the ring.
		if range > 0 then
			if dist > range then
				insideRing = false
				
				--Check if we're in a identical totem's ring.
				for guid, data in pairs(activeTotems) do 
					if data.dist and totemGUID ~= guid and data.totemName == totemData.totemName then
						if data.dist <= data.range then
							insideRing = true
							break
						end
					end
				end
			end
		end

		if totemData.minimapIcon and totemData.minimapIcon:IsShown() then
			self:UpdateMinimapLocation(totemGUID, range, dist, xDist, yDist, insideRing)
		end
		
		if self.db.profile.showTotemDistance == true and totemData.caster == self.userName then
			for i=1, 4 do 
				fName = "TotemFrameTotem"..i
				tFrame = _G[fName]
				if tFrame and tFrame:IsShown() and tFrame.slot == totemData.totemSlot then
					_G[fName.."Distance"]:SetText( self.db.profile.totemDistanceFormat:format(dist) )
					if self.db.profile.totemDistanceSmoothColour == true then
						_G[fName.."Distance"]:SetTextColor( self:GreenToRed(dist, range) )
					else
						if insideRing then
							_G[fName.."Distance"]:SetTextColor(0,1,0)
						else
							_G[fName.."Distance"]:SetTextColor(1,0,0)
						end
					end
					
					break
				end
			end
		end
		
	--~ 	if P.AVRdisplay == true then
	--~ 		local totemSlot = totemData.totemSlot or 1
	
	--~ 		if totemData.zoneMap == self:GetCurrentMap() then
	--~ 			self:UpdateAVRDisplay(totemGUID, dist, insideRing, totemSlot)
	--~ 		end
	--~ 	end
	end
end

------------------------------------------------------------------
function core:TotemAgeDeath(totemGUID)							--
-- Timer that fires when the totem would have died of old age.	--
------------------------------------------------------------------
	RemoveTotem(totemGUID)
end

do 
	local tonumber = tonumber
	local GetBuildInfo = GetBuildInfo
	local select = select
	local UnitIsUnit = UnitIsUnit
	local UnitInParty = UnitInParty
	local UnitInRaid = UnitInRaid
	
	local _, eventType, _, srcName, _, dstGUID, dstName, srcGUID
	local eventVars=10
	
	local tC, tZ, tX, tY
	local precise
	local totemSlot
	local zoneMap
	function core:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
		--timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags
--~ 		_, eventType, _, srcName, _, dstGUID, dstName  = ...-- ***
		
		if tonumber((select(4, GetBuildInfo()))) >= 40200 then
			_, eventType, _, _, srcName, _, _, dstGUID, dstName = ...
			eventVars = 12
		else
			_, eventType, _, srcName, _, dstGUID, dstName  = ...-- ***
		end
		
		
		
		if eventType == "SPELL_SUMMON" then
			local spellID = select(eventVars, ...)
			--8075 Strength of Earth totem
--~ 			Debug(event, "A", eventType, srcName, dstGUID, dstName, spellID)
			--spellID, spellName, spellSchool
			
			if totemInfo[spellID] then
				
				precise = false --Do we know the totem's coords, or the shaman's coords.
				totemSlot = totemInfo[spellID].slot
				
--~ 				Debug(event, "B", eventType, srcName, dstGUID, dstName, spellID)
				
				if UnitIsUnit("player", srcName) then
					tC, tZ, _, _, zoneMap = self:GetUnitPosition("player")
					
					--Get the totem's coords based on our current direction and the heading that totem usually spawns.
					tX, tY = self:GetTotemCoords(self.totemHeadings[totemSlot])
					precise = true
--~ 					Debug(event, "C", eventType, srcName, dstGUID, dstName, spellID)
				elseif P.trackOthersTotems == true and UnitInParty(srcName) or UnitInRaid(srcName) then
					tC, tZ, tX, tY, zoneMap = self:GetUnitPosition(srcName)
--~ 					Debug(event, "D", eventType, srcName, dstGUID, dstName, spellID)
				end
			
				if tC then
--~ 					Debug(event, "E", eventType, srcName, dstGUID, dstName, spellID)
					self:AddTotem(dstGUID, spellID, srcName, tC, tZ, tX, tY, precise, totemSlot, zoneMap)
					if P.broadcastTotemLocation == true and precise == true then
--~ 						Debug(event, "F", eventType, srcName, dstGUID, dstName, spellID)
						BroadcastTotemLoc(dstGUID, spellID, tC, tZ, tX, tY, zoneMap)
					end
	
				end
			end
			
		elseif eventType == "UNIT_DIED" or eventType == "UNIT_DESTROYED" or eventType == "UNIT_DISSIPATES" then
--~ 			Debug(event, eventType, srcName, dstGUID, dstName)
			
			if activeTotems[dstGUID] then
	--~ 			Debug("COMBAT", dstGUID, eventType)
				RemoveTotem(dstGUID)
				return
			end
			if dstName and activeCasters[dstName] then
				RemoveCastersTotems(dstName)
			end
		end
	end
end

do
	local unitID
	local UnitIsUnit = UnitIsUnit
	local UnitHealth = UnitHealth
	local GetUnitName = GetUnitName
	local name
	function core:UNIT_HEALTH(event, ...)
		unitID = ...
		
		if not UnitIsUnit(unitID,"player") and UnitHealth(unitID) == 0 then
			name = GetUnitName(unitID, true)
			if name and activeCasters[name] then
				RemoveCastersTotems(name)
			end
		end
	end
end


do 
	local success, totemGUID, spellID, tC, tZ, tX, tY, zoneMap
	function core:IncComm(prefix, message, distribution, sender)
		if sender ~= self.userName then
			if prefix == commPrefixTotemLoc then
				if P.trackOthersTotems == true then
					success, totemGUID, spellID, tC, tZ, tX, tY, zoneMap = core:Deserialize(message)
			--~ 		Debug("IncComm",sender.." dropped a totem!")
					
					if success then
						Debug("IncComm", sender, totemGUID, zoneMap, tX, tY)
						self:AddTotem(totemGUID, spellID, sender, tC, tZ, tX, tY, true, nil, zoneMap)
					end
				end
			elseif prefix == commPrefixRemoveTotem then
				success, totemGUID = core:Deserialize(message)
				if success then
					if activeTotems[totemGUID] and activeTotems[totemGUID].caster == sender then
	--~ 					Debug("IncComm","Removing "..sender.."'s totem.")
						RemoveTotem(totemGUID)
					end
				end
			end
		end
	end
end

do
	local data 
	------------------------------------------
	function core:HideTotem(totemGUID)		--
	-- Hide a totem but don't remove it.	--
	------------------------------------------
		data = activeTotems[totemGUID]
		if data then
			data.minimapIcon:Hide()
	--~ 		Debug("ZoneChanged", "Hiding ",totemGUID, self:GetCurrentMap(), data.zoneMap)
			
			--[[
			local meshes = self.totemMeshes[totemGUID]
			if meshes then
				for name, mesh in pairs(meshes) do 
					mesh.visible = false
				end
			end]]
		end
	end
end

do
	local data
	--------------------------------------
	function core:ShowTotem(totemGUID)	--
	-- Show a totem on minimap and AVR.	--
	--------------------------------------
		data = activeTotems[totemGUID]
		if data then
			data.minimapIcon:Show()
	--~ 		Debug("ZoneChanged", "Showing ",totemGUID)
			
			--[[
			local meshes = self.totemMeshes[totemGUID]
			if meshes then
				for name, mesh in pairs(meshes) do 
					mesh.visible = true
				end
			end]]
		end
	end
end

do
	local pairs = pairs
	local myMap
	------------------------------------------
	function core:ZoneChanged()				--
	-- Our zone has changed somehow. 		--
	-- Check if we should hide/show totems.	--
	------------------------------------------
		myMap = self:GetCurrentMap()
		for totemGUID, data in pairs(activeTotems) do 
			if data.zoneMap == myMap then
				self:ShowTotem(totemGUID)
			else
				self:HideTotem(totemGUID)
			end
		end
	end
end

do
	local GetPlayerMapPosition = GetPlayerMapPosition
	local data
	local tX, tY, zW, zH, pX, pY
	local dist
	--------------------------------------------------------------------------------------
	function core:CheckCasterDistance(totemGUID)										--
	-- Every ten seconds we check the distance of a caster and their totem's coords.	--
	-- If it's past 100 yards we remove the totem.										--
	--------------------------------------------------------------------------------------
		data = activeTotems[totemGUID]
		if data then
			pX, pY = GetPlayerMapPosition(data.caster)
			
			if pX then
				zW = data.zW --Zone width
				zH = data.zH --Zone height
			
				tX = data.tX * zW
				tY = data.tY * zH
	
				pX = pX * zW
				pY = pY * zH
				
				dist = self:ComputeDistance(tX, tY, pX, pY)
				
				if dist > 105 then
	--~ 				Debug("CheckCasterDistance", "Removing "..tostring(data.caster).."'s totem due to being out of range.", dist)
					RemoveTotem(totemGUID)
				end
			end
				
		else
	--~ 		Debug("CheckCasterDistance", totemGUID, "no longer exists.")
			self:CancelTimer(timers.distTimer[totemGUID], true)
			timers.distTimer[totemGUID] = nil
		end
	end
end


--	"TotemFrameTotem"..i.."Distance"
do 
	local frameName, frame
	local _G = _G
	
	function core:CreateTotemDistanceFrames()
		local d
		for i=1, 4 do 
			frameName = "TotemFrameTotem"..i
			frame = _G[frameName];
			if not _G[frameName.."Distance"] then
				d = frame:CreateFontString(frameName.."Distance", "BACKGROUND", "GameFontNormalSmall")
				d:SetText("0 y")
				d:SetPoint("TOP", _G[frameName.."Duration"], "BOTTOM")
				d:Hide()
			end
		end
	end
end
