local folder, core = ...

local LibStub = LibStub
local activeTotems = core.activeTotems
local Debug

--~ local pairs = pairs

local defaultSettings = core.defaultSettings
--~ local GetSpellInfo = GetSpellInfo
--~ local table_getn = table.getn
--~ local table_insert = table.insert
--~ local table_sort = table.sort

--~ local type = type
--~ local table_insert = table.insert
--~ local table_sort = table.sort

local L = core.L or LibStub("AceLocale-3.0"):GetLocale(folder, true)

defaultSettings.profile.trackOthersTotems = true
defaultSettings.profile.broadcastTotemLocation = true
defaultSettings.profile.colourRingBySchool = true
defaultSettings.profile.totemColour[1]	= {r=1, g=0, b=0} --red
defaultSettings.profile.totemColour[2]	= {r=0, g=1, b=0} --green
defaultSettings.profile.totemColour[3]	= {r=0, g=0.5, b=1} --cyan
defaultSettings.profile.totemColour[4]	= {r=0.6, g=0.4, b=0.8} --amethyst

defaultSettings.profile.minimapIconRange = 200
defaultSettings.profile.iconSize = 0
defaultSettings.profile.edgeIconSize = 16

defaultSettings.profile.insideRingColour	= {r=0, g=1, b=0} --green, 15% alpha
defaultSettings.profile.outsideRingColour	= {r=1, g=0, b=0} --red 
defaultSettings.profile.insideRingAlpha = 0.2 -- 20%
defaultSettings.profile.outsideRingAlpha = 1 -- 100%
defaultSettings.profile.shakeMinimapRings = true -- 100%


defaultSettings.profile.showOnMinimap = true
defaultSettings.profile.showTotemDistance = true
defaultSettings.profile.totemDistanceFormat = "%.1f y"
defaultSettings.profile.totemDistanceSmoothColour = false

local prev_OnInitialize = core.OnInitialize;
function core:OnInitialize()
	prev_OnInitialize(self)
	Debug = self.Debug
	
	self:AddBlizzardOptions()
	self:BuildAboutMenu()
end



local defaultShow = {}

local P
do
	local pairs = pairs
	local GetSpellInfo = GetSpellInfo
	
	local name
	local prev_OnEnable = core.OnEnable;
	function core:OnEnable()
		prev_OnEnable(self);
	
		P = self.db.profile
		
		
		for spellID, show in pairs(self.totemList) do
			name = GetSpellInfo(spellID)
			if name then
				defaultShow[name] = show
			end
		end
	--~ 	Debug("OnEnable","Options.lua")
		self:BuildPrefixUI()
	end
end

local name = function(info) 
	local key = info[#info]
	return L[key.."Name"] and L[key.."Name"]:format(core.db.profile[key]) or key
end
local desc = function(info) 
	local key = info[#info]
	return L[key.."Desc"] and L[key.."Desc"]:format(core.db.profile[key]) or key
end

function core:OpenOptionsFrame()
	LibStub("AceConfigDialog-3.0"):Open(self.title)
end

do
	local UnitName = UnitName
	local HideUIPanel = HideUIPanel
	local InterfaceOptionsFrame = InterfaceOptionsFrame
	local GameMenuFrame = GameMenuFrame
	local CreateFrame = CreateFrame
	local table_wipe = table.wipe
	local coreOpts
	
	local config, dialog, blizName
	function core:AddBlizzardOptions()
	-- I'm going to use AceConfigDialog's window to show my options. 
	-- But I still want my addon listed in Blizzard's options frame. 
	-- The only thing shown in the Blizzard options frame will be a button 
	-- to open AceConfigDialog's window.
	
		local blizOptions = {
			name = core.name,
			type = "group",
			args = {
			
				Desc = {
					type = "description",
					name = L.blizOptionsName:format(UnitName("player")),
					order = 1,
				},
				openOptionsFrame = {
					type = "execute",	order	= 2,
					name	= name, --L.openOptionsFrameName,
					desc	= desc, --L.openOptionsFrameDesc,
					func = function(info, v)
						HideUIPanel(InterfaceOptionsFrame)
						HideUIPanel(GameMenuFrame)
						
						--If I open the AceConfigDialog window now it will close once HideUIPanel fires. 
						-- So I'm making a frame to open it on the next frame refresh.
						local f = CreateFrame("Frame")
						f:SetScript("OnUpdate", function(this, elapsed) 
							core:OpenOptionsFrame()
							this:Hide()
							table_wipe(this)
						end)
					end,
				},
			}
		}
	
		config = LibStub("AceConfig-3.0")
		dialog = LibStub("AceConfigDialog-3.0")
		blizName = core.title.."bliz"
		
		config:RegisterOptionsTable(blizName, blizOptions )
		coreOpts = dialog:AddToBlizOptions(blizName, core.titleFull)
	end
end

do 
	local pairs = pairs
	core.CoreOptionsTable = {
		name = titleFull,
		type = "group",
		childGroups = "tab",
	
		get = function(info)
			local key = info[#info]
			return P[key]
		end,
		set = function(info, v)
			local key = info[#info] -- backup_enabled, etc...
			P[key] = v
			core:Disable()
			core:Enable()
		end,
	
		args = {
			core={
				name = L["Core"],
				type = "group",
				order = 1,
				args={
				
					enable = {
						type = "toggle",	order	= 1,
						name	= L["Enable"],
						desc	= L["Enables / Disables the addon"],
						set = function(info,val) 
							if val == true then
								core:Enable()
							else
								core:Disable()
							end
						end,
						get = function(info) return core:IsEnabled() end
					},
				
					trackOthersTotems = {
						type = "toggle",	order	= 2,
						name	= L["Track other's totems"],
						desc	= L["Add other shaman's totems to your minimap."],
						set = function(info,val) 
							P.trackOthersTotems = val
						end,
						get = function(info) return P.trackOthersTotems end
					},
				
					broadcastTotemLocation = {
						type = "toggle",	order	= 3,
						name	= L["Broadcast totem location"],
						desc	= L["Broadcast your totem's location to group members."],
						set = function(info,val) 
							P.broadcastTotemLocation = val
						end,
						get = function(info) return P.broadcastTotemLocation end
					},
					
	
					
					fireColour = {
						name = L["Fire Colour"],	order	= 10,
						desc = L["Colour of the circle around the totem"],
						type = "color",
						set = function(info,r,g,b,a) 
							P.totemColour[1].r = r
							P.totemColour[1].g = g
							P.totemColour[1].b = b
						end,
						get = function(info) return P.totemColour[1].r, P.totemColour[1].g, P.totemColour[1].b, P.totemColour[1].a end
					},
				
					earthColour = {
						name = L["Earth Colour"],	order	= 11,
						desc = L["Colour of the circle around the totem"],
						type = "color",
						set = function(info,r,g,b,a) 
							P.totemColour[2].r = r
							P.totemColour[2].g = g
							P.totemColour[2].b = b
						end,
						get = function(info) return P.totemColour[2].r, P.totemColour[2].g, P.totemColour[2].b, P.totemColour[2].a end
					},
	
					waterColour = {
						name = L["Water Colour"],	order	= 12,
						desc = L["Colour of the circle around the totem"],
						type = "color",
						set = function(info,r,g,b,a) 
							P.totemColour[3].r = r
							P.totemColour[3].g = g
							P.totemColour[3].b = b
							P.totemColour[3].a = a
						end,
						get = function(info) return P.totemColour[3].r, P.totemColour[3].g, P.totemColour[3].b, P.totemColour[3].a end
					},
				
				
					airColour = {
						name = L["Air Colour"],	order	= 13,
						desc = L["Colour of the circle around the totem"],
						type = "color",
						set = function(info,r,g,b,a) 
							P.totemColour[4].r = r
							P.totemColour[4].g = g
							P.totemColour[4].b = b
						end,
						get = function(info) return P.totemColour[4].r, P.totemColour[4].g, P.totemColour[4].b, P.totemColour[4].a end
					},
				
	
	
					insideRingColour = {
						name = L["Inside range colour"],	order	= 18,
						desc = L["Colour of the ring when you're inside."],
						type = "color",
						hasAlpha	= false,
						set = function(info,r,g,b,a) 
							P.insideRingColour.r = r
							P.insideRingColour.g = g
							P.insideRingColour.b = b
					--~ 		P.insideRingColour.a = a
						end,
						get = function(info) return P.insideRingColour.r, P.insideRingColour.g, P.insideRingColour.b, P.insideRingColour.a end --
					},
					
					
					outsideRingColour = {
						name = L["Outside range colour"],	order	= 20,
						desc = L["Colour of the ring when you're outside."],
						type = "color",
						hasAlpha	= false,
						set = function(info,r,g,b,a) 
							P.outsideRingColour.r = r
							P.outsideRingColour.g = g
							P.outsideRingColour.b = b
					--~ 		P.outsideRingColour.a = a
							
						end,
						get = function(info) return P.outsideRingColour.r, P.outsideRingColour.g, P.outsideRingColour.b, P.outsideRingColour.a end
					},
					
					showTotemDistance = {
						name = name,
						desc = desc,
						type = "toggle",
						order	= 21,
					},
	
					totemDistanceFormat = {
						type = "input",	order	= 22,
						name = name,
						desc = desc,
					},
	
					totemDistanceSmoothColour = {
						name = name,
						desc = desc,
						type = "toggle",
						order	= 23,
					},
	
				}
			},
			
			minimap={
				name = L["Minimap"],
				type = "group",
				order = 3,
				args={
	
					showOnMinimap = {
						type = "toggle",	order	= 1,
						name	= L["Show on minimap"],
						desc	= L["Show totem rings on minimap."],
						set = function(info,val) 
							P.showOnMinimap = val
							
							if val == true then
								core.updateThrottle = 0.05
								
								core:ShowAllMinimapRings()
							else
								core.updateThrottle = 0.25
								
								core:HideAllMinimapRings()
							end
						end,
						get = function(info) return P.showOnMinimap end
					},
				
					colourRingBySchool = {
						type = "toggle",	order	= 2,
						name	= L["Colour ring by school"],
						desc	= L["Colour the rings on minimap by totem school instead of range"],
						set = function(info,val) 
							P.colourRingBySchool = val
							
							if val == true then
								local totemSlot
								for totemGUID, data in pairs(activeTotems) do 
									totemSlot = data.totemSlot
									if data.icon then
										data.icon.outlineTexture:SetVertexColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, P.totemColour[totemSlot].a)
									end
								end
							end
							
						end,
						get = function(info) return P.colourRingBySchool end
					},
				
					minimapIconRange = {
						type	= "range",	order	= 3,
						name = L["Minimap Show Range"],
						desc = L["Hide the icon if we're beyond * yards away."],
						min		= 50,
						max		= 200,
						step	= 5,
						set = function(info,val) 
							P.minimapIconRange = val
							core:ForceUpdate()
						end,
						get = function(info) return P.minimapIconRange end
					},
	
					iconSize = {
						type	= "range",	order	= 4,
						name = L["Icon Size"],
						desc = L["Size of the totem icon on the minimap. (Not the circle)"],
						min		= 0,
						max		= 32,
						step	= 4,
						set = function(info,val) 
							P.iconSize = val;
							core:UpdateMiniampSpellSize()
							core:ForceUpdate()
						end,
						get = function(info) return P.iconSize end
					},
				
					edgeIconSize = {
						type	= "range",	order	= 5,
						name = L["Edge Icon Size"],
						desc = L["Size of the totem icon when on minimap edge"],
						min		= 0,
						max		= 32,
						step	= 4,
						set = function(info,val) 
							P.edgeIconSize = val
							core:ForceUpdate()
						end,
						get = function(info) return P.edgeIconSize end
					},
				
				
				
					insideRingAlpha = {
						type	= "range",	order	= 6,
						name = L["Inside ring transparency"],
						desc = L["When inside, change ring transparency."],
						min		= 0,
						max		= 1,
						step	= 0.05,
						isPercent	= true,
						set = function(info,val) 
							P.insideRingAlpha = val
							core:ForceUpdate()
						end,
						get = function(info) return P.insideRingAlpha end
					},
	
					
					outsideRingAlpha = {
						type	= "range",	order	= 7,
						name = L["Outside ring transparency"],
						desc = L["When outside, change ring transparency."],
						min		= 0,
						max		= 1,
						step	= 0.05,
						isPercent	= true,
						set = function(info,val) 
							P.outsideRingAlpha = val
							core:ForceUpdate()
						end,
						get = function(info) return P.outsideRingAlpha end
					},
					
	
					
					shakeMinimapRings = {
						type = "toggle",	order	= 8,
						name	= L["Shake Minimap Rings"],
						desc	= L["When you move outside a ring, shake it briefly."],
						set = function(info,val) 
							P.shakeMinimapRings = val
						end,
						get = function(info) return P.shakeMinimapRings end
					},
					
				},
			},
		},
	}
end

do
	local pairs = pairs
	local GetSpellInfo = GetSpellInfo
	local table_getn = table.getn
	local table_insert = table.insert
	local table_sort = table.sort
	local type = type
	
	local totemName
	function core:BuildPrefixUI()--	/script TR.BuildPrefixUI()
	--~ 	Debug("BuildPrefixUI A")
		self.CoreOptionsTable.args.totems = { --reset
			name = L["Totems"],
			type = "group",
			order = 2,
			args={}
		}
		
		
		local totems = {} --sortable list
		
		
		for spellID, data in pairs(self.totemInfo) do 
			totemName = data.name or GetSpellInfo(spellID)
			if table_getn(totems) == 0 then
				table_insert(totems, #totems+1, totemName)
			else
				for i=1, table_getn(totems) do 
					if totems[i] == totemName then
						break
					elseif i == table_getn(totems) then
						table_insert(totems, #totems+1, totemName)
					end
				end
			end
			
		end
	--~ 	Debug("BuildPrefixUI B")
		table_sort(totems, function(a,b) 
			if(a and b) then 
				return a < b
			end 
		end)
	--~ 	Debug("BuildPrefixUI C")
			--totemOpts
		for i=1, table_getn(totems) do 
			totemName = totems[i]
	--~ 		Debug("BuildPrefixUI", i, totems[i])
			
			
			if P.totemOpts[totemName] == nil then
				P.totemOpts[totemName] = {shown = defaultShow[totemName] or 1} --show by default, using a table incase I need to add options later.
			end
			
			if type(P.totemOpts[totemName].shown) == "boolean" then --old show/hide option.
				P.totemOpts[totemName].shown = defaultShow[totemName] or 1 
			end
			
			self.CoreOptionsTable.args.totems.args[totemName] = {
				type = "select",	order	= i,
				
				name = totemName,
				desc = L["Track %s's radius"]:format(totemName),
	
				values = function()
					return {
						[1] = L["Raid totems"],
						[2] = L["Party totems"],
						[3] = L["My totem"],
						[4] = L["Never"],
					}
					
				end,
				set = function(info,val) 
					local totemName = info[2]
				
					P.totemOpts[totemName].shown = val
				end,
				get = function(info) return P.totemOpts[info[2]] and P.totemOpts[info[2]].shown end
			}
	
			
		end
		
	--~ 	Debug("BuildPrefixUI D")
	end
end

local GetFireNovaTotems
do 
	local pairs = pairs
	local table_insert = table.insert
	local table_sort = table.sort
	local table_wipe = table.wipe
	
	local totems = {}
	function GetFireNovaTotems()
		table_wipe(totems)

		for totemName in pairs(P.showFireNova) do 
			table_insert(totems, #totems+1, totemName)
		end
		table_sort(totems)
		
		return totems
	end
end


do
	local tostring = tostring
	local GetAddOnMetadata = GetAddOnMetadata
	local pairs = pairs
	local fields = {"Author", "X-Category", "X-License", "X-Email", "Email", "eMail", "X-Website", "X-Credits", "X-Localizations", "X-Donate", "X-Bitcoin"}
	local haseditbox = {["X-Website"] = true, ["X-Email"] = true, ["X-Donate"] = true, ["Email"] = true, ["eMail"] = true, ["X-Bitcoin"] = true}
	local fNames = {
		["Author"] = L.author,
		["X-License"] = L.license,
		["X-Website"] = L.website,
		["X-Donate"] = L.donate,
		["X-Email"] = L.email,
		["X-Bitcoin"] = L.bitcoinAddress,
	}
	local yellow = "|cffffd100%s|r"
	
	local val
	local options
	function core:BuildAboutMenu()
		options = self.options
		
		self.CoreOptionsTable.args.about = {
			type = "group",
			name = L.about,
			order = 99,
			args = {
			}
		}

		
		self.CoreOptionsTable.args.about.args.title = {
			type = "description",
			name = yellow:format(L.title..": ")..self.title,
			order = 1,
		}
		self.CoreOptionsTable.args.about.args.version = {
			type = "description",
			name = yellow:format(L.version..": ")..self.version,
			order = 2,
		}
		self.CoreOptionsTable.args.about.args.notes = {
			type = "description",
			name = yellow:format(L.notes..": ")..tostring(GetAddOnMetadata(folder, "Notes")),
			order = 3,
		}
	
		for i,field in pairs(fields) do
			val = GetAddOnMetadata(folder, field)
			if val then
				
				if haseditbox[field] then
					self.CoreOptionsTable.args.about.args[field] = {
						type = "input",
						name = fNames[field] or field,
						order = i+10,
						desc = L.clickCopy,
						width = "full",
						get = function(info)
							local key = info[#info]
							return GetAddOnMetadata(folder, key)
						end,	
					}
				else
					self.CoreOptionsTable.args.about.args[field] = {
						type = "description",
						name = yellow:format((fNames[field] or field)..": ")..val,
						width = "full",
						order = i+10,
					}
				end
		
			end
		end
	
		LibStub("AceConfig-3.0"):RegisterOptionsTable(self.title, self.CoreOptionsTable ) --
		LibStub("AceConfigDialog-3.0"):SetDefaultSize(self.title, 600, 500) --680
	end
end